<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Api\ApiController;
use App\Http\Services\Api\SeminarSignUpService;
use App\Http\Requests\Api\SeminarSignUp\StoreRequest;

use Log;

class SeminarSignUpController extends ApiController
{
    
    private $seminarSignUpSv;
    
    public function __construct()
    {
        $this->seminarSignUpSv = new SeminarSignUpService();

    }

    // save data to db
    public function insertData(StoreRequest $request)
    {
        $trackNo = $request->input('trackNo', '');
        $lang = substr($trackNo, 0, 2);
        $firstName_orig = $request->input('firstName', '');
        $firstName = $this->safeEncrypt($firstName_orig, 'arm');

        $lastName = $this->safeEncrypt($request->input('lastName', ''), 'arm');
        $companyName = $this->safeEncrypt($request->input('companyName', ''), 'arm');

        $companyEmail_orig = $request->input('companyEmail', '');
        $companyEmail = $this->safeEncrypt($companyEmail_orig, 'arm');
        
        $backupEmail = $request->input('backupEmail', '');
        if (!is_null($backupEmail)) {
        
            $backupEmail_orig = $request->input('backupEmail', '');
            $backupEmail = $this->safeEncrypt($backupEmail_orig, 'arm');
        } else {
            $backupEmail_orig = '';
        }

        $phoneNumber = $this->safeEncrypt($request->input('phoneNumber', ''), 'arm');
        $country = $request->input('country', '');
        $registeredSession = $request->input('registeredSession', '');
        $lunchOptions = $request->input('lunchOptions', '');
        $typeOfIndustry = $request->input('typeOfIndustry', '');
        $typeOfJob = $request->input('typeOfJob', '');
        $jobTitle = $request->input('jobTitle', '');
        $trackOfInterest = $request->input('trackOfInterest', '');
        $areaOfInterest = $request->input('areaOfInterest', '');
        $howToKnowAboutTheEvent = $request->input('howToKnowAboutTheEvent', '');
        $consentAcceptEmail = $request->input('consentAcceptEmail', '');
        $consentPrivacyPolicy = $request->input('consentPrivacyPolicy', '');
        
        $overOrNot = $this->seminarSignUpSv->overLimitOrNot($trackNo);

        if ($overOrNot) {

            $this->seminarSignUpSv->insertData(
                $firstName, 
                $lastName, 
                $companyName, 
                $companyEmail, 
                $backupEmail, 
                $phoneNumber, 
                $country, 
                $trackNo, 
                $registeredSession,
                $lunchOptions,
                $typeOfIndustry,
                $typeOfJob,
                $jobTitle,
                $trackOfInterest,
                $areaOfInterest,
                $howToKnowAboutTheEvent,
                $consentAcceptEmail,
                $consentPrivacyPolicy,
            );
            $res = '報名成功';

            if ($lang=='TW') {
                $this->mailToUser_TW($firstName_orig, $companyEmail_orig, $backupEmail_orig);
            } elseif ($lang=='JP') {
                $this->mailToUser_JP($firstName_orig, $companyEmail_orig, $backupEmail_orig);
            } elseif ($lang=='KR') {
                $this->mailToUser_KR($firstName_orig, $companyEmail_orig, $backupEmail_orig);
            } else {
                $this->mailToUser_EN($firstName_orig, $companyEmail_orig, $backupEmail_orig);
            }
            
        } else {

            $res = '已達報名上限';
        }
        $data = [
            'res' => $res,
        ];
        
        return $this->apiResponse($data);
    }

    public function getData()
    {
        $list = $this->seminarSignUpSv->getData();
        
        $data = [
            'list' => $list
        ];
        
        return $this->apiResponse($data);
    }

    /**
     * 參數加解密模組: 加密部分，建議使用環境變數中的 secret key 作加解密種子
     */
    public function safeEncrypt(string $message, string $skey): string
    {
        // if (mb_strlen($key, '8bit') !== SODIUM_CRYPTO_SECRETBOX_KEYBYTES) {
        //     throw new RangeException('Key is not the correct size (must be 32 bytes).');
        // }
        // $nonce = random_bytes(SODIUM_CRYPTO_SECRETBOX_NONCEBYTES);
        
        // $cipher = base64_encode(
        //     $nonce .
        //     sodium_crypto_secretbox(
        //         $message,
        //         $nonce,
        //         $key
        //     )
        // );
        // sodium_memzero($message);
        // sodium_memzero($key);
        // return $cipher;
        $strArr = str_split(base64_encode($message));
        $strCount = count($strArr);
        foreach (str_split($skey) as $key => $value)
        $key < $strCount && $strArr[$key].=$value;
        return str_replace(array('=', ' ', '/'), array('O0O0O', 'o000o', 'oo00o'), join('', $strArr));
    }
    
    /**
     * 參數加解密模組: 解密部分，建議使用環境變數中的 secret key 作加解密種子
     */
    public function safeDecrypt(string $encrypted, string $skey): string
    {
        // $decoded = base64_decode($encrypted);
        // $nonce = mb_substr($decoded, 0, SODIUM_CRYPTO_SECRETBOX_NONCEBYTES, '8bit');
        // $ciphertext = mb_substr($decoded, SODIUM_CRYPTO_SECRETBOX_NONCEBYTES, null, '8bit');
        
        // $plain = sodium_crypto_secretbox_open(
        //     $ciphertext,
        //     $nonce,
        //     $key
        // );
        // if (!is_string($plain)) {
        //     throw new Exception('Invalid MAC');
        // }
        // sodium_memzero($ciphertext);
        // sodium_memzero($key);
        
        // return $plain;
        $strArr = str_split(str_replace(array('O0O0O', 'o000o', 'oo00o'), array('=', ' ', '/'), $encrypted), 2);
        $strCount = count($strArr);
        foreach (str_split($skey) as $key => $value)
        $key <= $strCount && isset($strArr[$key]) && $strArr[$key][1] === $value && $strArr[$key] = $strArr[$key][0];
        return base64_decode(join('', $strArr));
    }
    
    public function mailToUser_TW(string $firstName='', string $companyEmail='', string $backupEmail='')
    {
        $data = array('name'=>$firstName);

        if ($backupEmail) {
            \Mail::send(['text'=>'mailTW'], $data, function($message) use ($companyEmail, $firstName, $backupEmail) {
                $message->to($companyEmail, $firstName)->subject('報名成功');       // Registration is complete
                $message->cc($backupEmail, $firstName)->subject('報名成功');       // Registration is complete
                $message->from('no-reply@erapr.com.tw','Arm Tech Symposia 2');
            });
        } else {
            \Mail::send(['text'=>'mailTW'], $data, function($message) use ($companyEmail, $firstName) {
                $message->to($companyEmail, $firstName)->subject('報名成功');       // Registration is complete
                $message->from('no-reply@erapr.com.tw','Arm Tech Symposia 2');
            });
        }
    }

    public function mailToUser_JP(string $firstName='', string $companyEmail='', string $backupEmail='')
    {
        $data = array('name'=>$firstName);

        if ($backupEmail) {
            \Mail::send(['text'=>'mailJP'], $data, function($message) use ($companyEmail, $firstName, $backupEmail) {
                $message->to($companyEmail, $firstName)->subject('登録完了');
                $message->cc($backupEmail, $firstName)->subject('登録完了');
                $message->from('no-reply@erapr.com.tw','Arm Tech Symposia 2');
            });
        } else {
            \Mail::send(['text'=>'mailJP'], $data, function($message) use ($companyEmail, $firstName) {
                $message->to($companyEmail, $firstName)->subject('登録完了');
                $message->from('no-reply@erapr.com.tw','Arm Tech Symposia 2');
            });
        }
    }

    public function mailToUser_KR(string $firstName='', string $companyEmail='', string $backupEmail='')
    {
        $data = array('name'=>$firstName);

        if ($backupEmail) {
            \Mail::send(['text'=>'mailKR'], $data, function($message) use ($companyEmail, $firstName, $backupEmail) {
                $message->to($companyEmail, $firstName)->subject('등록이 완료되었습니다');
                $message->cc($backupEmail, $firstName)->subject('등록이 완료되었습니다');
                $message->from('no-reply@erapr.com.tw','Arm Tech Symposia 2');
            });
        } else {
            \Mail::send(['text'=>'mailKR'], $data, function($message) use ($companyEmail, $firstName) {
                $message->to($companyEmail, $firstName)->subject('등록이 완료되었습니다');
                $message->from('no-reply@erapr.com.tw','Arm Tech Symposia 2');
            });
        }
    }
    
    public function mailToUser_EN(string $firstName='', string $companyEmail='', string $backupEmail='')
    {
        $data = array('name'=>$firstName);

        if ($backupEmail) {
            \Mail::send(['text'=>'mailEN'], $data, function($message) use ($companyEmail, $firstName, $backupEmail) {
                $message->to($companyEmail, $firstName)->subject('Registration is complete');
                $message->cc($backupEmail, $firstName)->subject('Registration is complete');
                $message->from('no-reply@erapr.com.tw','Arm Tech Symposia 2');
            });
        } else {
            \Mail::send(['text'=>'mailEN'], $data, function($message) use ($companyEmail, $firstName) {
                $message->to($companyEmail, $firstName)->subject('Registration is complete');
                $message->from('no-reply@erapr.com.tw','Arm Tech Symposia 2');
            });
        }
    }

}


