<?php

namespace App\Http\Controllers\Backend\DataManagement;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use App\Http\Services\CheckParamService;
use App\Http\Controllers\Controller;
use App\Http\Services\Backend\DataManagement\TrackManagementService;
use Log;

class TrackManagementController extends Controller
{
    
    // 相關私有服務層調用器宣告
    private $checkParamSv;
    private $trackManagementSv;
    
    public function __construct()
    {
        // 建構服務層調用器
        $this->checkParamSv=new CheckParamService();
        $this->trackManagementSv = new TrackManagementService();
    }
    public function index()
    {
        // 渲染
        return view('admin.DataManagement.TrackManagement');

    }
    public function grid()
    {
        // 取得參數
        $param = $_GET;
        if ($param == null) exit();
        $draw = $param["draw"]; //客戶端傳來的查詢次數，無條件回傳用以核對
        $orderColumn = $param["order"][0]["column"] + 1; //前端從 0 開始送，但 mysql 從 1 開始算
        $orderDir = $param["order"][0]["dir"];
        $start = $param["start"];   // 頁碼
        $length = $param["length"]; // 一頁多大
        $searchValue = $param["search"]["value"];
        //客製化搜尋欄位
        $keyword = $param["columns"][1]["search"]["value"] ?? null ;

        // 驗證
        if ($keyword != filter_var($keyword, FILTER_SANITIZE_SPECIAL_CHARS)) $keyword = "___CANNOT_FIND_STRING___";
        if (!$this->checkParamSv->LenMToN($keyword, 0, 50)) $keyword = "___CANNOT_FIND_STRING___";
        
        //資料庫
        $recordsTotal = 0;

        $result=$this->trackManagementSv->getList($recordsTotal, $orderColumn, $orderDir, $start, $length, $searchValue, $keyword);

        // 整理返回資料
        $data = array();
        $registeredSession = '';
        $lunchOptions = '';
        for ($i = 0; $i < count($result); $i++) {
            
            $result[$i]["ctrEdit"]=' 
                <a class="btn _btn-warning btn-sm _fz-12" href="track/edit/'.$result[$i]["id"].'"><span class="_glyphicon _glyphicon-list"></span> 編輯 </a>
                <a class="btn btn-danger btn-sm _fz-12" onclick="return delWarning();" href="track/delete/'.$result[$i]["id"].'"><span class="_glyphicon _glyphicon-remove"></span> 刪除 </a>';

            $data[] = array(
                //一般資料
                $result[$i]["id"],
                htmlspecialchars($result[$i]["trackNo"]),
                htmlspecialchars($result[$i]["trackTitle"]),
                htmlspecialchars($result[$i]["trackLimit"]),
                $result[$i]["createDate"],
                $result[$i]["ctrEdit"],
            );
        }
        $json = array(
            "draw"            => $draw,
            "recordsTotal"    => $recordsTotal,
            "recordsFiltered" => $recordsTotal, //其實還是填入所有筆數，本次筆數可從陣列取得
            "data"            => $data,
        );
        
        // 返回
        return json_decode(json_encode($json), true);
    }

    /**
     * 新增訊息頁面
     *   本端點非 API，直接到 view
     * @return array:          返回渲染頁
     */
    public function create()
    {
        // 取得參數與驗證
        // 服務層取得資料(以及實現各種業務上的邏輯)
        
        $tagUnassign = $this->tagManagementSv->getTagByReferral("", false);
        //有一個referral 貼多個標的情況
        $tagUnassign_uniques = array();
        foreach ($tagUnassign as $c) {
            $tagUnassign_uniques[$c["id"].'-'.$c["tagName"]] = $c; // Get unique country by code.
        }
        
        $tagAssigned = $this->tagManagementSv->getTagByReferral("", true);
        //有一個referral 貼多個標的情況
        $tagAssigned_uniques = array();
        foreach ($tagAssigned as $c) {
            $tagAssigned_uniques[$c["id"].'-'.$c["tagName"]] = $c; // Get unique country by code.
        }
        $referralList = $this->referralSv->getReferral();
        $referral=array();
        foreach($referralList as $data)
        {
            array_push($referral,$data["referral"]);
        }
        // 渲染
        return view('admin.DataManagement.ReferralManagementEdit', [
            'operdata'        => "",
            'richmenuId'         => 0,
            'richmenus'          => $this->richmenuManagementSv->getRichmenuList(),
            'tagUnassign'        => $tagUnassign_uniques, 
            'tagAssigned'        => $tagAssigned_uniques,
            'existReferral'        => $referral,
        ]);
    }

    /**
     * 編輯訊息頁面
     *   本端點非 API，依照變量取得資料後，渲染到 view
     *
     * @param integer $id :    訊息的序號
     *
     * @return array:          返回渲染頁
     */
    public function edit($id)
    {
        // 取得參數與驗證
        // 服務層取得資料(以及實現各種業務上的邏輯)
        $res = $this->trackManagementSv->getTrackDataById($id);

        // 渲染
        return view('admin.DataManagement.TrackManagementEdit', [
            'operdata'      => $res,
            'id'            => $id,
        ]);
    }
    
    /**
     * 新增或編輯標籤的端點
     *   本端點非 API，依照參數新增或修改資料後，跳轉到既有的 view
     *
     * @param Request $request :  整包參數
     *
     * @return array:             返回跳轉渲染頁的資訊
     */
    public function store(Request $request)
    {
        // 取得參數(由於是內部系統因此不做後端驗證)
        \Log::info($request->all());
        // return;
        $mode = $request->operateMode;
        $trackNo = $request->trackNo;
        $trackTitle = $request->trackTitle;
        $trackLimit = $request->trackLimit;
        // $id = ($request->mode == 'insert') ? '' : $request->id;
        // $name = $request->name;
        // $referral = $request->referral;
        // $orlCode = $request->orlCode;
        // $richmenuId = $request->richmenuId;
        // $hidTag = $request->hidTag;
        // $folderS3 = 'urlQRCode'; //要儲存到S3上的folder名稱
        /**
         * picsee短網址
         */
        // 服務層設置(以及實現各種業務上的邏輯)
        if ($mode='insert') {
            // 新增模式
            // 直接創建一組短網址
            //$orlCode = $this->insertOrl($name, env('LINE_LIFF_URL').'/'.env('LIFF_PAGE_LOGIN').'?referral='.$referral.'&liffId='.env('LIFF_PAGE_LOGIN'), $name);
            // $orlCode = $this->insertOrl($name, env('APP_URL').'/web/referrallogin/'.$referral, $name);
            /**
             * 改掉以往存到ORL的方式，改以透過Picsee來產生短網址
             * 增加儲存欄位
             * @picseeCode 短網址存取碼
             * @picseeUrl 短網址
             * @qRCode_pic
             */
            //新的新增至PICSEE
            // $picseeData = $this->insertPicsee(env("APP_FRONTEND_URL").'/web/referrallogin/'.$referral);

            // $picseeUrl = str_replace(env("PICSEE_SERCH_DOMAIN"),env("PICSEE_REPLACE_DOMAIN"),$picseeData->picseeUrl);
            // $explodeGetCode = explode("/",$picseeUrl);
            // $picseeCode = end($explodeGetCode);
            // $qRCode = $this->makeQRCode($picseeCode,$picseeUrl,$folderS3);
            // $id = $this->trackManagementSv->insertReferral(
            //     $name,
            //     $referral,
            //     $richmenuId,
            //     $picseeCode,
            //     $picseeUrl,
            //     $qRCode,
            //     $orlCode
            // );
        } elseif ($mode=='edit') {
//             $referralData = $this->referralSv->getReferralById($id);
//             // 編輯模式
//             if ($orlCode != 0) {
//                 $this->updateOrl($orlCode, $name, $name);
//             } else {
//                 // 針對導入短網址服務之後，但又已經存在的紀錄，開放只要編輯過就可以取得短網址
//                 //$orlCode = $this->insertOrl($name, env('LINE_LIFF_URL').'/'.env('LIFF_PAGE_LOGIN').'?referral='.$referral.'&liffId='.env('LIFF_PAGE_LOGIN'), $name);
// //                $orlCode = $this->insertOrl($name, env('APP_URL').'/web/referrallogin/'.$referral, $name);
//                 //此段取得短網址的功能調整指向picsee流程
//                 if($referralData["picseeCode"] == ""){
//                     $picseeData = $this->insertPicsee(env("APP_URL").'/web/referrallogin/'.$referral);
//                     $picseeUrl = str_replace(env("PICSEE_SERCH_DOMAIN"),env("PICSEE_REPLACE_DOMAIN"),$picseeData->picseeUrl);
//                     $explodeGetCode = explode("/",$picseeUrl);
//                     $picseeCode = end($explodeGetCode);
//                     $qRCode = $this->makeQRCode($picseeCode,$picseeUrl,$folderS3);
//                     $this->referralSv->modifyReferralPicsee(
//                         $id,
//                         $picseeCode,
//                         $picseeUrl,
//                         $qRCode
//                     );
//                 }
//             }
            $this->trackManagementSv->store(
                $id,
                $trackNo,
                $trackTitle,
                $trackLimit,
            );
        }
        // $this->referralTagManagementSv->modifyreferralTagByReferralAndTagIds($id, $hidTag);
    
        // 跳轉
        return redirect('/backend/dataManagement/track');
    }
    
}